/*
@(#)NegativeBinomialDistribution.java

Copyright (C) 2001-2009  Kyle Siegrist, Dawn Duehring
Department of Mathematical Sciences
University of Alabama in Huntsville

This program is part of Virtual Laboratories in Probability and Statistics,
http://www.math.uah.edu/stat/, a project partially supported by the
National Science Foundation under grant number DUE-0089377.

This program is licensed under a Creative Commons License. Basically, you are free to copy,
distribute, and modify this program, and to make commercial use of the program.
However you must give proper attribution.
See http://creativecommons.org/licenses/by/2.0/ for more information.
*/

package edu.uah.math.distributions;
import java.io.Serializable;

/**
* This class models the negative binomial distribution with specified successes
* parameter and probability parameter. Version TRIAL governs the trial number of the specified success
* number in a sequence of Bernoulli trials. Version COUNT governs the number of failures before the
* specified number of successes.
* @author Kyle Siegrist
* @author Dawn Duehring
* @version December, 2005
*/
public class NegativeBinomialDistribution extends Distribution implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//Constants
	public final static int TRIAL = 0, COUNT = 1;
	//Paramters
	private int successes, version = TRIAL;
	private double probability;

	/**
	* This general constructor creates a new negative binomial distribution with
	* given parameter values.
	* @param k the number of successes
	* @param p the probability of success
	* @param v the version
	*/
	public NegativeBinomialDistribution(int k, double p, int v){
		setParameters(k, p, v);
	}

	/**
	* This special constructor creates a new negative binomial distribution with
	* specified success and probability parameter, and with default version TRIAL.
	*/
	public NegativeBinomialDistribution(int k, double p){
		this(k, p, TRIAL);
	}

	/**
	* This default constructor creates a new negative binomial distribution with
	* successes parameter 1 and probability parameter 0.5.
	*/
	public NegativeBinomialDistribution(){
		this(1, 0.5);
	}

	/**
	* This method set the paramters and the default domain.
	* @param k the number of successes
	* @param p the probability of success
	* @param v the version
	*/
	public void setParameters(int k, double p, int v){
		int lowerBound, upperBound;
		//Correct for invalid parameters
		if(k < 1) k = 1;
		if(p <= 0) p = 0.05;
		if(p > 1) p = 1;
		//Assign parameters
		successes = k;
		probability = p;
		version = v;
		//Set truncated values
		if (version == TRIAL) lowerBound = successes;
		else lowerBound = 0;
		upperBound = (int)Math.ceil(getMean() + 4 * getSD());
		setDomain(lowerBound, upperBound, 1, DISCRETE);
	}

	/**
	* This method sets the successes and probability parameters.
	* @param k the number of successes
	* @param p the probability of success
	*/
	public void setParameters(int k, double p){
		setParameters(k, p, version);
	}

	/**
	* This method set the successes parameter.
	* @param k the number of successes
	*/
	public void setSuccesses(int k){
		setParameters(k, probability, version);
	}

	/**
	* This method returns the successes parameter.
	* @return the number of successes
	*/
	public int getSuccesses(){
		return successes;
	}

	/**
	* This method returns the probability parameter.
	* @return the probability of success
	*/
	public double getProbability(){
		return probability;
	}

	/**
	* This method sets the probability parameter.
	* @param p the probability of success
	*/
	public void setProbability(double p){
		setParameters(successes, p, version);
	}

	/**
	* This method computes the probability density function.
	* @param x a number in the domain of the distribution
	* @return the probability density at x
	*/
	public double getDensity(double x){
		int n = (int)Math.rint(x);
		if (version == TRIAL) return Functions.comb(n - 1, successes - 1) * Math.pow(probability, successes) * Math.pow(1 - probability, n - successes);
		else return Functions.comb(n + successes - 1, successes - 1) * Math.pow(probability, successes)	* Math.pow(1 - probability, n);
	}

	/**
	* This method computes the maximum value of the density function.
	* @return the maximum value of the probability density function
	*/
	public double getMaxDensity(){
		double mode;
		mode = Math.floor((successes - 1) / probability + 1);
		if (version == COUNT) mode = mode - successes;
		return getDensity(mode);
	}

	/**
	* This method computes the mean of the distribution.
	* @return the mean
	*/
	public double getMean(){
		if (version == TRIAL) return successes / probability;
		else return successes / probability - successes;
	}

	/**
	* This method computes the variance of the distribution.
	* @return the variance
	*/
	public double getVariance(){
		return (successes * (1 - probability)) / (probability * probability);
	}

	/**
	* This method computes the probability generating function.
	* @param t a real number
	* @return the probability generating function at t
	*/
	public double getPGF(double t){
		double r = 1 / (1 - probability), pgf;
		if (version == TRIAL) pgf = Math.pow(t * probability / (1 - t + t * probability), successes);
		else pgf = Math.pow(probability / (1 - t + t * probability), successes);
		if(Math.abs(t) < r) return pgf;
		else if (t >= r) return Double.POSITIVE_INFINITY;
		else return Double.NaN;
	}

	/**
	* This method computes moment generating function.
	* @param t a real number
	* @return the moment generating function
	*/
	public double getMGF(double t){
		return getPGF(Math.exp(t));
	}

	/**
	* This method simulates a value from the distribution, overriding the
	* correspondin default method in Distribution.
	* @return a simulated value from the distribution
	*/
	public double simulate(){
		int count = 0, trials = 0;
		while (count <= successes){
			if (Math.random() < probability) count++;
			trials++;
		}
		if (version == TRIAL) return trials;
		else return trials - successes;
	}

	/**
	* This method returns a string that gives the name of the distribution and the values of
	* the parameters.
	* @return a string giving the name of the distribution and the values of the parameters
	*/
	public String toString(){
		return "Negative binomial distribution [successes = " + successes
			+ ", probability = " + probability + "]";
	}
}

