/*
@(#)PolyaDistribution.java

Copyright (C) 2001-2009  Kyle Siegrist, Dawn Duehring
Department of Mathematical Sciences
University of Alabama in Huntsville

This program is part of Virtual Laboratories in Probability and Statistics,
http://www.math.uah.edu/stat/, a project partially supported by the
National Science Foundation under grant number DUE-0089377.

This program is licensed under a Creative Commons License. Basically, you are free to copy,
distribute, and modify this program, and to make commercial use of the program.
However you must give proper attribution.
See http://creativecommons.org/licenses/by/2.0/ for more information.
*/

package edu.uah.math.distributions;
import java.io.Serializable;

/**
* This class models the Polya distribution with specified parameters.
* This distribution governs the number of red balls in a sample of a specified size
* in Polya's urn scheme.
* @author Kyle Siegrist
* @version June, 2006
*/
public class PolyaDistribution extends Distribution implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//Variables
	private int red, green, step, trials;

	/**
	* This general constructor creates the Polya distribution with specified parameters.
	* @param a the initial number of red balls
	* @param b the initial number of green balls
	* @param s the step size (the number of new balls added)
	* @param n the sample size
	*/
	public PolyaDistribution(int a, int b, int s, int n){
		setParameters(a, b, s, n);
	}

	/**
	* This default constructor creates the Polya distribution with 10 trials
	* and with the other parameters 1.
	*/
	public PolyaDistribution(){
		this(1, 1, 1, 10);
	}

	/**
	* This method sets the parameters and computes the default domain.
	*/
	public void setParameters(int a, int b, int c, int n){
		//Correct invalid parameters
		if (a < 1) a = 1;
		if (b < 1) b = 1;
		if (n < 1) n = 1;
		red = a; green = b; step = c; trials = n;
		setDomain(0, trials, 1, DISCRETE);
	}

	/**
	* This method sets the number of red balls.
	* @param a the number of red balls
	*/
	public void setRed(int a){
		setParameters(a, green, step, trials);
	}

	/**
	* This method returns the number of red balls.
	* @return the number of trials
	*/
	public int getRed(){
		return red;
	}

	/**
	* This method sets the number of green balls.
	* @param b the number of green balls
	*/
	public void setGreen(int b){
		setParameters(red, b, step, trials);
	}

	/**
	* This method returns the number of green balls.
	* @return the number of green balls
	*/
	public int getGreen(){
		return green;
	}

	/**
	* This method sets the step size.
	* @param s the step size
	*/
	public void setStep(int s){
		setParameters(red, green, s, trials);
	}

	/**
	* This method returns the step size.
	* @return the number of step size.
	*/
	public int getStep(){
		return step;
	}

	/**
	* This method sets the number of trails.
	* @param n the number of trials
	*/
	public void setTrials(int n){
		setParameters(red, green, step, n);
	}

	/**
	* This method returns the number of trials.
	* @return the number of trials
	*/
	public int getTrials(){
		return trials;
	}

	/**
	* This method computes the probability  density function.
	* @param x a number in the domain of the distribuiton
	* @return the probability density at x
	*/
	public double getDensity(double x){
		int population = red + green;
		int k = (int)Math.rint(x);
		double product1, product2;
		if (k < 0 | k > trials) return 0;
		else {
			product1 = 1;
			for (int i = 0; i < k; i++)
				product1 = product1 * (trials - i) * (red + i * step) / ((population + i * step) * (k - i));
			product2 = 1;
			for (int j = 0; j < trials - k; j++)
				product2 = product2 * (green + j * step) / (population + (k + j) * step);
			return product1 * product2;
		}
	}

	/**
	* This method computes the mean of the distribution.
	* @return the mean
	*/
	public double getMean(){
		double probability = (double)red / (red + green);
		return trials * probability;
	}

	/**
	* This method computes the variance of the distributioin
	* @return the variance
	*/
	public double getVariance(){
		double probability = (double)red / (red + green);
		double factor = (double)step / (red + green + step);
		return trials * probability * (1 - probability) * (1 + (trials - 1) * factor);
	}

	/**
	* This method simulates the a value from the distribution. This is done by
	* simulating the Bernoulli trials and counting the number of successes.
	* @return a simulated value from the distribution
	*/
	public double simulate(){
		int successes = 0, totalRed = red, totalGreen = green;
		double probability;
		for (int i = 1; i <= trials; i++){
			probability = (double)totalRed / (totalRed + totalGreen);
			if (Math.random() < probability) {
				successes++;
				totalRed = totalRed + step;
			}
			else totalGreen = totalGreen + step;
		}
		return successes;
	}

	/**
	* This method returns a string that gives the name of the distribution and the values of
	* the parameters.
	* @return a string giving the name of the distribution and the values of the parameters
	*/
	public String toString(){
		return "Polya distribution [red = " + red
			+ ", green = " + green + ", step = " + step + ", trials = " + trials + 	"]";
	}

}

