/*
@(#)BivariateNormalExperiment.java

Copyright (C) 2001-2009  Kyle Siegrist, Dawn Duehring
Department of Mathematical Sciences
University of Alabama in Huntsville

This program is part of Virtual Laboratories in Probability and Statistics,
http://www.math.uah.edu/stat/, a project partially supported by the
National Science Foundation under grant number DUE-0089377.

This program is licensed under a Creative Commons License. Basically, you are free to copy,
distribute, and modify this program, and to make commercial use of the program.
However you must give proper attribution.
See http://creativecommons.org/licenses/by/2.0/ for more information.
*/

package edu.uah.math.experiments;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.io.Serializable;
import edu.uah.math.distributions.NormalDistribution;
import edu.uah.math.distributions.RandomVariable;
import edu.uah.math.distributions.Domain;
import edu.uah.math.devices.RecordTable;
import edu.uah.math.devices.StatisticsTable;
import edu.uah.math.devices.Parameter;
import edu.uah.math.devices.BivariateScatterPlot;
import edu.uah.math.devices.RandomVariableGraph;
import edu.uah.math.devices.RandomVariableTable;

/**
* This experiment generates a point (X, Y) from a bivariate normal distribution with
* mean (0, 0). The point is shown graphically as a red dot in the scatterplot, and the
* coordinates are recorded on each update. The distribution regression line is shown in
* blue in the scatterplot, and the sample regression line is shown in red. The density
* function and moments of X and of Y are shown in the distribution graphs, and the moments
* are given in the distribution tables. The last table gives the distribution and sample
* correlation. The distribution standard deviations and correlations can be varied with
* scroll bars.
* @author Kyle Siegrist
* @author Dawn Duehring
* @version August, 2003
*/
public class BivariateNormalExperiment extends Experiment implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	double sigmaX = 1; double sigmaY = 1, sum, distCor;
	private NormalDistribution xDist = new NormalDistribution(0, 1), yDist = new NormalDistribution(0,1);
	RecordTable recordTable = new RecordTable(new String[] {"Run", "X", "Y"});
	RandomVariable x = new RandomVariable(xDist, "X");
	RandomVariable y = new RandomVariable(yDist, "Y");
	RandomVariableGraph xGraph = new RandomVariableGraph(x);
	RandomVariableGraph yGraph = new RandomVariableGraph(y);
	RandomVariableTable xTable = new RandomVariableTable(x);
	RandomVariableTable yTable = new RandomVariableTable(y);
	BivariateScatterPlot xyScatter = new BivariateScatterPlot(new Domain(-6, 6, 1, Domain.CONTINUOUS));
	StatisticsTable xyTable = new StatisticsTable("(X, Y)", new String[]{"Cov", "Cor", "Slope", "Intercept"});
	JToolBar toolBar = new JToolBar("Parameter Toolbar");
	Parameter sigmaXScroll = new Parameter(0.5, 2, 0.1, 1, "Standard deviation of X", "\u03c3x");
	Parameter sigmaYScroll = new Parameter(0.5, 2, 0.1, 1, "Standard deviation of Y", "\u03c3y");
	Parameter rhoScroll = new Parameter(-1, 1, 0.01, 0, "Correlation", "\u03c1");

	/**
	* This method initializes the experiment, including the scatterplot, parameter scrollbars
	* labels, random variable graphs, and the correlation table.
	*/
	public void init(){
		super.init();
		setName("Bivariate Normal Experiment");
		//Sliders
		sigmaXScroll.applyDecimalPattern("0.0");
		sigmaXScroll.getSlider().addChangeListener(this);
		sigmaYScroll.applyDecimalPattern("0.0");
		sigmaYScroll.getSlider().addChangeListener(this);
		rhoScroll.applyDecimalPattern("0.00");
		rhoScroll.getSlider().addChangeListener(this);
		//Toolbar
		toolBar.setLayout(new FlowLayout(FlowLayout.LEFT));
		toolBar.add(sigmaXScroll);
		toolBar.add(sigmaYScroll);
		toolBar.add(rhoScroll);
		addToolBar(toolBar);
		//Scatterplot
		xyScatter.setParameters(0, 0);
		xyScatter.setToolTipText("(X, Y) scatterplot");
		xyScatter.setMinimumSize(new Dimension(100, 100));
		addComponent(xyScatter, 0, 0, 1, 1);
		//xGraph
		xGraph.setMinimumSize(new Dimension(100, 100));
		addComponent(xGraph, 1, 0, 1, 1);
		//yGraph
		yGraph.setMinimumSize(new Dimension(100, 100));
		addComponent(yGraph, 2, 0, 1, 1);
		//Record table
		recordTable.setDescription("(X, Y): sample point");
		addComponent(recordTable, 0, 1, 1, 2);
		//X Table
		addComponent(xTable, 1, 1, 1, 1);
		//Y Table
		addComponent(yTable, 2, 1, 1, 1);
		//(X, Y) Table
		xyTable.setDescription("Covariance, correlation, regression slope and intercept of (X, Y)");
		addComponent(xyTable, 1, 2, 2, 1);
		//Final actions
		validate();
		reset();
	}

	/**
	* This method returns basic information about the applet, including copyright
	* information, descriptive information, and instructions.
	* @return applet infomration
	*/
	public String getAppletInfo(){
		return super.getAppletInfo() + "\n\n"
			+ "Visit http://www.math.uah.edu/stat/applets/BivariateNormalExperiment.xhtml for more information\n"
			+ "about the applet and for a mathematical discussion of the bivariate normal experiment.";
	}

	/**
	* This method defines the experiment.  A point from the bivariate normal distribution is
	* simulated and passed to the scatterplot.
	*/
	public void doExperiment(){
		double r, theta, u, v;
		super.doExperiment();
		//Simulate a point from the bivariate normal distribution*/
		r = Math.sqrt(-2 * Math.log(Math.random()));
		theta = 2 * Math.PI * Math.random();
		u = r * Math.cos(theta);
		r = Math.sqrt(-2 * Math.log(Math.random()));
		theta = 2 * Math.PI * Math.random();
		v = r * Math.cos(theta);
		x.setValue(sigmaX * u);
		y.setValue(sigmaY * distCor * u + sigmaY * Math.sqrt(1 - distCor * distCor) * v);
		sum = sum + x.getValue() * y.getValue();
		//Add the point
		xyScatter.addPoint(x.getValue(), y.getValue());
	}

	/**
	* This method updates the experiment, including the random variable graphs, correlation
	* table, and record table.
	*/
	public void update(){
		super.update();
		int runs = getTime();
		//Compute the sample correlation and covariance*/
		double sampleCov = (sum -  runs * x.getIntervalData().getMean() * y.getIntervalData().getMean()) / (runs - 1);
		double sampleCor = sampleCov / (x.getIntervalData().getSD() * y.getIntervalData().getSD());
		double sampleSlope = sampleCov / x.getIntervalData().getVariance();
		double sampleIntercept = y.getIntervalData().getMean() - sampleSlope * x.getIntervalData().getMean();
		recordTable.addRecord(new double[]{getTime(), x.getValue(), y.getValue()});
		xyTable.setDataValues(new double[]{sampleCov, sampleCor, sampleSlope, sampleIntercept});
		//Update tables and graphs
		xyScatter.setStatistics(sampleSlope, sampleIntercept);
		xyScatter.repaint();
		xGraph.repaint();
		yGraph.repaint();
		xTable.repaint();
		yTable.repaint();
	}

	/**
	* This method resets the experiment, including the random variable graphs and tables,
	* the correlation table, scatterplot and record table.
	*/
	public void reset(){
		super.reset();
		recordTable.reset();
		sum = 0;
		x.reset();
		y.reset();
		xyScatter.reset();
		xGraph.reset();
		yGraph.reset();
		xTable.reset();
		yTable.reset();
		xyTable.reset();
	}

	/**
	* This method handles the scrollbar events that occur when parameters are changed.
	* @param e the change event
	*/
	public void stateChanged(ChangeEvent e){
		double distCov, distSlope;
		if (e.getSource() == sigmaXScroll.getSlider()){
			sigmaX = sigmaXScroll.getValue();
			xDist.setParameters(0, sigmaX);
		}
		else if (e.getSource() == sigmaYScroll.getSlider()){
			sigmaY = sigmaYScroll.getValue();
			yDist.setParameters(0, sigmaY);
		}
		else if (e.getSource() == rhoScroll.getSlider()) distCor = rhoScroll.getValue();
		distCov = distCor * sigmaX * sigmaY;
		distSlope = distCor * sigmaY / sigmaX;
		xyScatter.setParameters(distSlope, 0);
		xyTable.setDistributionValues(new double[]{distCov, distCor, distSlope, 0});
		reset();
	}
}


