/*
@(#)EhrenfestExperiment.java

Copyright (C) 2005-2009  Kyle Siegrist
Department of Mathematical Sciences
University of Alabama in Huntsville

This program is part of Virtual Laboratories in Probability and Statistics,
http://www.math.uah.edu/stat/, a project partially supported by the
National Science Foundation under grant number DUE-0089377.

This program is licensed under a Creative Commons License. Basically, you are free to copy,
distribute, and modify this program, and to make commercial use of the program.
However you must give proper attribution.
See http://creativecommons.org/licenses/by/2.0/ for more information.
*/

package edu.uah.math.experiments;
import java.awt.FlowLayout;
import java.awt.event.MouseEvent;
import java.awt.Color;
import javax.swing.JToolBar;
import javax.swing.JComboBox;
import javax.swing.event.ChangeEvent;
import java.awt.event.ItemEvent;
import java.awt.Dimension;
import edu.uah.math.distributions.Domain;
import edu.uah.math.distributions.BinomialDistribution;
import edu.uah.math.distributions.RandomVariable;
import edu.uah.math.devices.RecordTable;
import edu.uah.math.devices.Timeline;
import edu.uah.math.devices.RandomVariableGraph;
import edu.uah.math.devices.RandomVariableTable;
import edu.uah.math.devices.Parameter;

/**
* This class models the Ehrenfest Markov chain. The model consists of two urns with a
* total of m balls.  In the basic Ehrenfest chain, at each discrete time unit, a ball is
* chosen at random and moved to the other urn.  In the modified Ehrenfest chain, at each
* discrete time unit, a ball is chosen at random, an urn is chosen at random, and the
* chosen ball is placed in the chosen urn.  The number of balls m can be varied. The
* initial state can be specified by clicking on a state  when the time is 0. The graph
* and table show the proportion of time spent in each state. As time increases, these
* proportions converge to the steady-state probabilities of the chain.
* @author Dawn Duehring
* @author Kyle Siegrist
* @version December, 2005
*/
public class EhrenfestExperiment extends Experiment{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//Constants
	public final static int BASIC = 0, MODIFIED = 1;
	//Variables
	private int balls = 10, model = BASIC, initialState = 0, currentState = 0;
	//Objects
	private RecordTable recordTable = new RecordTable(new String[] {"Time", "X"});
	private Timeline chain = new Timeline(new Domain(0, balls, 1, Domain.DISCRETE), "x");
	private JToolBar toolBar = new JToolBar("Parameter Toolbar");
	private BinomialDistribution limitDistribution = new BinomialDistribution(balls, 0.5);
	private RandomVariable state = new RandomVariable(limitDistribution, "X");
	private RandomVariableGraph stateGraph = new RandomVariableGraph(state);
	private RandomVariableTable stateTable = new RandomVariableTable(state);
	private JComboBox modelChoice = new JComboBox();
	//Scrollbars and labels
	private Parameter ballScroll = new Parameter(1, 100, 1, balls, "Number of balls", "m");

	/**
	* This method initialize the experiment, including the toolbar, record table, Markov chain,
	* histogram, and data table.
	*/
	public void init(){
		super.init();
		setName("Ehrenfest Experiment");
		//Scrollbar
		ballScroll.getSlider().addChangeListener(this);
		//Combox box
		modelChoice.addItemListener(this);
		modelChoice.addItem("Basic");
		modelChoice.addItem("Modified");
		modelChoice.setToolTipText("Model");
		//Toolbar
		toolBar.setLayout(new FlowLayout(FlowLayout.LEFT));
		toolBar.add(modelChoice);
		toolBar.add(ballScroll);
		addToolBar(toolBar);
		//Urn Chain
		chain.setToolTipText("Markov chain: at time 0, click to set the initial state");
		chain.setPointSize(6);
		chain.setMargins(35, 20, 20, 20);
		chain.setMinimumSize(new Dimension(100, 30));
		chain.addMouseListener(this);
		chain.setCurrentTimeColor(Color.red);
		addComponent(chain, 0, 0, 2, 1, 10, 0);
		//Random variable graph
		addComponent(stateGraph, 0, 1, 2, 1);
		//Record Table
		recordTable.setDescription("X: state");
		addComponent(recordTable, 0, 2, 1, 1);
		//Data Table
		addComponent(stateTable, 1, 2, 1, 1);
		//Final actions
		validate();
		reset();
	}

	/**
	* This method returns basic information about the applet, including copyright
	* information, descriptive information, and instructions.
	* @return applet infomration
	*/
	public String getAppletInfo(){
		return super.getAppletInfo() + "\n\n"
			+ "Visit http://www.math.uah.edu/stat/applets/EhrenfestExperiment.xhtml for more information\n"
			+ "about the applet and for a mathematical discussion of the Ehrenfest experiment.";
	}

	/**
	* This method handles the scrollbar events for changing the number of states.
	* @param e the change event
	*/
	public void stateChanged(ChangeEvent e){
	    if (e.getSource() == ballScroll.getSlider()){
			balls = (int)ballScroll.getValue();
			chain.setDomain(new Domain(0, balls, 1, Domain.DISCRETE));
			limitDistribution.setParameters(balls, 0.5);
			if (initialState > balls) initialState = balls;
			reset();
		 }
	}

	/**
	* This method handles events associated with the choice of the Ehrenfest model,
	* @param e the item event
	*/
	public void itemStateChanged(ItemEvent e){
		if(e.getSource() == modelChoice){
			model = modelChoice.getSelectedIndex();
			reset();
		}
		else super.itemStateChanged(e);
	}

	/**
	* This method sets the initial state when the user clicks on a ball.
	* @param e the mouse event
	*/
	public void mouseClicked(MouseEvent e){
		if (e.getSource() == chain && getTime()  == 0){
			int x = (int)Math.rint(chain.getXScale(e.getX()));
			if (x < 0) x = 0;
			else if (x > balls) x = balls;
			initialState = x;
			reset();
		}
	}

	/**
	* This method performs the next step in the process. A note is played, depending on
	* the state.
	*/
	public void step(){
		doExperiment();
		playNote(currentState);
		update();
	}

	/**
	* This method resets the experiment, including the Markov chain, the record table
	* the histogram and data table.
	*/
	public void reset(){
		super.reset();
		currentState = initialState;
		chain.resetData();
		chain.addTime(currentState, Color.red);
		chain.setCurrentTime(currentState);
		recordTable.reset();
		recordTable.addRecord(new double[]{getTime(), initialState});
		state.reset();
		stateGraph.reset();
		stateTable.reset();
	}

	/**
	* This method defines the experiment. The Markov chain moves to the next state.
	*/
	public void doExperiment(){
		super.doExperiment();
		move();

	}

	/**
	* This method updates the experiment, including the record table, Markov chain,
	* histogram, and data table.
	*/
	public void update(){
		super.update();
		recordTable.addRecord(new double[]{getTime(), currentState});
		chain.resetData();
		chain.addTime(initialState, Color.blue);
		chain.addTime(currentState, Color.red);
		chain.setCurrentTime(currentState);
		stateGraph.repaint();
		stateTable.repaint();
	}

	/**
	* This method sets the transition probabilities
	*/
	public void move(){
		double m = (double)balls;
		double p = Math.random();
		if (model==BASIC){
			if (p <= currentState / m) currentState = currentState - 1;
			else currentState = currentState + 1;
		}
		else{
			if (p <= currentState / (2 * m)) currentState = currentState - 1;
			else if (currentState / (2 * m) + 0.5 < p) currentState = currentState + 1;
		}
		state.setValue(currentState);
	}

}
