/*
@(#)RouletteExperiment.java

Copyright (C) 2001-2009  Kyle Siegrist, Dawn Duehring
Department of Mathematical Sciences
University of Alabama in Huntsville

This program is part of Virtual Laboratories in Probability and Statistics,
http://www.math.uah.edu/stat/, a project partially supported by the
National Science Foundation under grant number DUE-0089377.

This program is licensed under a Creative Commons License. Basically, you are free to copy,
distribute, and modify this program, and to make commercial use of the program.
However you must give proper attribution.
See http://creativecommons.org/licenses/by/2.0/ for more information.
*/

package edu.uah.math.experiments;
import java.io.Serializable;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ActionEvent;
import javax.swing.JComboBox;
import javax.swing.JToolBar;
import javax.swing.Timer;
import edu.uah.math.devices.RecordTable;
import edu.uah.math.devices.RouletteWheel;
import edu.uah.math.devices.RandomVariableGraph;
import edu.uah.math.devices.RandomVariableTable;
import edu.uah.math.distributions.FiniteDistribution;
import edu.uah.math.distributions.RandomVariable;

/**
* This class models the casino roulette experiment.
* @author Kyle Siegrist
* @author Dawn Duehring
* @version August, 2003
*/
public class RouletteExperiment extends Experiment implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//Constants
	public final static int ONE = 0, TWO = 1, THREE = 2, FOUR = 3, SIX = 4, TWELVE = 5, EIGHTEEN = 6;
	//Variables
	private int score, spinIndex, scoreIndex, spinCount, win, betType = ONE;
	private double[] prob;
	//Objects
	private RecordTable recordTable = new RecordTable(new String[]{"Run", "X", "W"});
	private RouletteWheel wheel = new RouletteWheel();
	private JComboBox betChoice = new JComboBox();
	private JToolBar toolBar = new JToolBar("Parmaeter Toolbar");
	private FiniteDistribution dist = new FiniteDistribution();
	private RandomVariable winRV = new RandomVariable(dist, "W");
	private RandomVariableGraph winGraph = new RandomVariableGraph(winRV);
	private RandomVariableTable winTable = new RandomVariableTable(winRV);
	private Timer timer = new Timer(10, this);

	/**
	* This method initializes the experiment by setting up the toolbar with the bet
	* choice, and initializing the roulette wheel.
	*/
	public void init(){
		super.init();
		setName("Roulette Experiment");
		//Bet choice
		betChoice.addItemListener(this);
		betChoice.setToolTipText("Bet");
		betChoice.addItem("Bet on 1");
		betChoice.addItem("Bet on 1, 2");
		betChoice.addItem("Bet on 1, 2, 3");
		betChoice.addItem("Bet on 1, 2, 4, 5");
		betChoice.addItem("Bet on 1-6");
		betChoice.addItem("Bet on 1-12");
		betChoice.addItem("Bet on 1-18");
		//toolbars
		toolBar.setLayout(new FlowLayout(FlowLayout.LEFT));
		toolBar.add(betChoice);
		addToolBar(toolBar);
		//Graphs
		addComponent(wheel, 0, 0, 1, 1);
		winGraph.setMinimumSize(new Dimension(250, 150));
		addComponent(winGraph, 1, 0, 1, 1);
		//Tables
		recordTable.setDescription("X: wheel score, W: win");
		addComponent(recordTable, 0, 1, 1, 1);
		winTable.setMinimumSize(new Dimension(250, 150));
		addComponent(winTable, 1, 1, 1, 1);
		//Final actions
		setDistribution();
		validate();
		reset();
	}

	/**
	* This method returns basic information about the applet, including copyright
	* information, descriptive information, and instructions.
	* @return applet information
	*/
	public String getAppletInfo(){
		return super.getAppletInfo() + "\n\n"
			+ "Visit http://www.math.uah.edu/stat/applets/RouletteExperiment.xhtml for more information\n"
			+ "about the applet and for a mathematical discussion of the roulette experiment";
	}

	/**
	* This method defines the exeperiment.  The wheel is spun and the result of the
	* bet determined.
	*/
	public void doExperiment(){
		super.doExperiment();
		score = wheel.spin();
		evaluate();
		winRV.setValue(win);
	}

	/**
	* This method runs the experiment one time and then plays a sound, depending on the
	* outcome.
	*/
	public void step(){
		stop();
		spinCount = 0;
		spinIndex = 0;
		score = wheel.spin();
		scoreIndex = wheel.getScoreIndex();
		wheel.setScoreIndex(spinIndex);
		wheel.setBallDrawn(true);
		timer.start();
	}

	/**
	* This method starts run mode, by first turning off the step timer if necessary.
	*/
	public void run(){
		timer.stop();
		super.run();
	}

	/**
	* This method stops the experiment, by first stopping the step timer and then calling the usual
	* stop method.
	*/
	public void stop(){
		timer.stop();
		super.stop();
	}

	/**
	* This method resets the experiment, including the roulette wheel, the random
	* variable graph and table, and the record table.
	*/
	public void reset(){
		timer.stop();
		super.reset();
		recordTable.reset();
		wheel.setBallDrawn(false);
		winRV.reset();
		winGraph.reset();
		winTable.reset();
	}


	/**
	* This method updates the experiment, including the roulette wheel, the record
	* table and the random variable graph and table.
	*/
	public void update(){
		super.update();
		wheel.setBallDrawn(true);
		recordTable.addRecord(new double[]{getTime(), score, win});
		winGraph.repaint();
		winTable.repaint();
	}


	/**
	* This method sets the distribution when the bet has changed.
	*/
	public void itemStateChanged(ItemEvent event){
		if (event.getSource() == betChoice){
			betType = betChoice.getSelectedIndex();
			setDistribution();
		}
		else super.itemStateChanged(event);
	}

	/**
	* This method handles the events associated with the step timer.
	* @param e the event
	*/
	public void actionPerformed(ActionEvent e){
		if (e.getSource() == timer){
			//The experiment is complete. Stop spinning and evaluate
			if (spinCount >= 111 & spinIndex == scoreIndex) {
				timer.stop();
				super.doExperiment();
				evaluate();
				winRV.setValue(win);
				update();
				playNote(win + 1);
			}
			//Keep spinning
			else{
				spinIndex++;
				if (spinIndex == 38) spinIndex = 0;
				wheel.setScoreIndex(spinIndex);
				spinCount++;
				wheel.setBallDrawn(true);
			}
		}
		else super.actionPerformed(e);
	}

	/**
	* This method sets the distribution of the win random variable.
	*/
	public void setDistribution(){
		switch(betType){
		case ONE:
			prob = new double[37];
			prob[0] = 37.0 / 38; prob[36] = 1.0 / 38;
			dist.setParameters(-1, 35, 1, prob);
			break;
		case TWO:
			prob = new double[19];
			prob[0] = 36.0 / 38; prob[18] = 2.0 / 38;
			dist.setParameters(-1, 17, 1, prob);
			break;
		case THREE:
			prob = new double[13];
			prob[0] = 35.0 / 38; prob[12] = 3.0 / 38;
			dist.setParameters(-1, 11, 1, prob);
			break;
		case FOUR:
			prob = new double[10];
			prob[0] = 34.0 / 38; prob[9] = 4.0 / 38;
			dist.setParameters(-1, 8, 1, prob);
			break;
		case SIX:
			prob = new double[7];
			prob[0] = 32.0 / 38; prob[6] = 6.0 / 38;
			dist.setParameters(-1, 5, 1, prob);
			break;
		case TWELVE:
			prob = new double[4];
			prob[0] = 26.0 / 38; prob[3] = 12.0 / 38;
			dist.setParameters(-1, 2, 1, prob);
			break;
		case EIGHTEEN:
			prob = new double[3];
			prob[0] = 20.0 / 38; prob[2] = 18.0 / 38;
			dist.setParameters(-1, 1, 1, prob);
			break;
		}
		reset();
	}

	/**
	* This method evaluates win random variable.
	*/
	public void evaluate(){
		switch(betType){
		case ONE:
			if (score == 1) win = 35;
			else win = -1;;
			break;
		case TWO:
			if (score == 1 | score == 2) win = 17;
			else win = -1;
			break;
		case THREE:
			if (score >=1 & score <= 3) win = 11;
			else win = -1;
			break;
		case FOUR:
			if (score == 1 | score == 2 | score == 4 | score == 5) win = 8;
			else win = -1;
			break;
		case SIX:
			if (score >= 1 & score <= 6) win = 5;
			else win = -1;
			break;
		case TWELVE:
			if (score >= 1 & score <= 12) win = 2;
			else win = -1;
			break;
		case EIGHTEEN:
			if (score >= 1 & score <= 18) win = 1;
			else win = -1;
			break;
		}
	}
}

