/*
@(#)SecretaryGame.java

Copyright (C) 2001-2009  Kyle Siegrist, Dawn Duehring
Department of Mathematical Sciences
University of Alabama in Huntsville

This program is part of Virtual Laboratories in Probability and Statistics,
http://www.math.uah.edu/stat/, a project partially supported by the
National Science Foundation under grant number DUE-0089377.

This program is licensed under a Creative Commons License. Basically, you are free to copy,
distribute, and modify this program, and to make commercial use of the program.
However you must give proper attribution.
See http://creativecommons.org/licenses/by/2.0/ for more information.
*/

package edu.uah.math.experiments;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionEvent;
import javax.swing.event.ChangeEvent;
import java.io.Serializable;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import edu.uah.math.distributions.Functions;
import edu.uah.math.distributions.IntervalData;
import edu.uah.math.distributions.Domain;
import edu.uah.math.devices.RecordTable;
import edu.uah.math.devices.Ball;
import edu.uah.math.devices.Urn;
import edu.uah.math.devices.Histogram;
import edu.uah.math.devices.DataTable;
import edu.uah.math.devices.Parameter;

/**
* This class models the secretary problem
* @author Kyle Siegrist
* @version October 2006
*/
public class SecretaryGame extends Game implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//Variables
	private int n = 10, candidate = 0, best = 0, runs = 0;
	private int[] sample = new int[n];
	//Objects
	private RecordTable recordTable = new RecordTable(new String[]{"Game", "X", "Y", "W"});
	private Parameter nScroll = new Parameter(2, 20, 1, n, "Number of candidates", "n");
	private Urn urn1 = new Urn(n, 26);
	private Urn urn2 = new Urn(n, 26);
	private JButton gameButton = new JButton();
	private JButton acceptButton = new JButton();
	private JButton rejectButton = new JButton();
	private IntervalData win = new IntervalData(new Domain(0, 1, 1, Domain.DISCRETE), "W");
	private Histogram winGraph = new Histogram(win);
	private DataTable winTable = new DataTable(win);

	/**
	* This method initializes the game, including the toolbar buttons, doors,
	* images.
	*/
	public void init(){
		super.init();
		setName("Marriage Game");
		//Slider
		nScroll.getSlider().addChangeListener(this);
		//Game button
		gameButton.addActionListener(this);
		gameButton.setIcon(new ImageIcon(getClass().getResource("step.png")));
		gameButton.setToolTipText("New game");
		//Accept button
		acceptButton.addActionListener(this);
		acceptButton.setIcon(new ImageIcon(getClass().getResource("CheckGreen.png")));
		acceptButton.setToolTipText("Accept");
		//Reject button
		rejectButton.addActionListener(this);
		rejectButton.setIcon(new ImageIcon(getClass().getResource("CrossRed.png")));
		rejectButton.setToolTipText("Reject");
		//Toolbars
		addTool(gameButton);
		addTool(rejectButton);
		addTool(acceptButton);
		addTool(nScroll);
		//Urn
		urn1.setMinimumSize(new Dimension(100, 40));
		addComponent(urn1, 0, 0, 3, 1, 10, 0);
		urn2.setMinimumSize(new Dimension(100, 40));
		addComponent(urn2, 0, 1, 3, 1, 10, 0);
		//Record Table
		recordTable.setDescription("X: best candidate, Y: selected candidate, W: player wins");
		addComponent(recordTable, 0, 2, 1, 1);
		//Win Table
		winTable.setStatisticsType(0);
		winTable.setDistributionType(DataTable.REL_FREQ);
		addComponent(winTable, 1, 2, 1, 1);
		//Win Graph
		winGraph.setMinimumSize(new Dimension(150, 150));
		winGraph.setStatisticsType(0);
		winGraph.setMargins(35, 20, 20, 20);
		addComponent(winGraph, 2, 2, 1, 1);
		//Final Actions
		validate();
		reset();
	}

	/**
	* This method returns basic information about the applet, including copyright
	* information, descriptive information, and instructions.
	* @return applet information
	*/
	public String getAppletInfo(){
		return super.getAppletInfo() + "\n\n"
			+ "Visit http://www.math.uah.edu/stat/applets/MarraigeGame.xhtml for more information\n"
			+ "about the applet and for a mathematical discussion of the marraige game.";
	}

	/**
	* This message resets the game by closing the doors and resetting the text message,
	* win graph, win table, and record table.
	*/
	public void reset(){
		super.reset();
		acceptButton.setEnabled(false);
		rejectButton.setEnabled(false);
		recordTable.reset();
		urn1.setDrawn(false);
		urn2.setDrawn(false);
		win.reset();
		winGraph.repaint();
		winTable.reset();
		runs = 0;
	}

	/**
	* This method handles the events for the Play button.  The doors are closed,
	* the host's first choice made, and the appropriate message displayed.
	*/
	public void actionPerformed(ActionEvent event){
		int rank = 0, x = 0, w = 0;
		Ball ball;
		if (event.getSource() == gameButton){
			urn1.setDrawn(false);
			urn2.setDrawn(false);
			acceptButton.setEnabled(true);
			rejectButton.setEnabled(true);
			candidate = 0;
			sample = Functions.getSample(n, n, Functions.WITHOUT_REPLACEMENT);
			ball = urn1.getBall(0);
			ball.setBallColor(Color.blue);
			ball.setValue(1);
			ball.setDrawn(true);

		}
		else if (event.getSource() == rejectButton){
			ball = urn1.getBall(candidate);
			ball.setBallColor(Color.red);
			ball.setTextColor(Color.white);
			ball.setDrawn(true);
			candidate++;
			if (candidate < n){
				rank = candidate + 1;
				for (int i = 0; i < candidate; i++){
					ball = urn1.getBall(i);
					if (sample[candidate] < sample[i]){
						x = ball.getValue();
						if (x < rank) rank = x;
					}
				}
				for (int j = 0; j < candidate; j++){
					ball = urn1.getBall(j);
					x = ball.getValue();
					if (rank <= x) ball.setValue(x + 1);
					ball.setDrawn(true);
				}
				ball = urn1.getBall(candidate);
				ball.setBallColor(Color.blue);
				ball.setValue(rank);
				ball.setDrawn(true);
			}
			playNote(6);
			if (candidate == n-1) rejectButton.setEnabled(false);
		}
		else if (event.getSource() == acceptButton){
			ball = urn1.getBall(candidate);
			ball.setBallColor(Color.green);
			ball.setDrawn(true);
			for (int i = 0; i < n; i++){
				ball = urn2.getBall(i);
				ball.setValue(sample[i]);
				if (ball.getValue() == 1){
					best = i;
					ball.setBallColor(Color.green);
				}
				else ball.setBallColor(Color.red);
				ball.setDrawn(true);
				acceptButton.setEnabled(false);
				rejectButton.setEnabled(false);
			}
			if (best == candidate) w = 1;
			else w = 0;
			runs++;
			playNote(w);
			win.setValue(w);
			winGraph.repaint();
			winTable.repaint();
			recordTable.addRecord(new double[]{runs, best + 1, candidate + 1, w});
		}
		else super.actionPerformed(event);
	}

	/**
	* The method handles the scrollbar events for adjusting the size
	* of the random permutation.
	* @param e the change event
	*/
	public void stateChanged(ChangeEvent e){
		if (e.getSource() == nScroll.getSlider()){
			n = (int)nScroll.getValue();
			urn1.setBallCount(n);
			urn2.setBallCount(n);
			reset();
		}
	}

}
